@@ -42,6 +42,7 @@ from lxml import etree
 
 from PyTango import DevFailed
 
+from taurus.external.ordereddict import OrderedDict
 from taurus.core.util.log import Logger
 from taurus.core.util.codecs import  CodecFactory
 
@@ -59,6 +60,10 @@ from sardana.macroserver.msexception import UnknownMacroLibrary, \
     LibraryError, UnknownMacro, MissingEnv, AbortException, StopException, \
     MacroServerException, UnknownEnv
 
+# These classes are imported from the "client" part of sardana, if finally
+# both the client and the server side needs them, place them in some common location 
+from sardana.taurus.core.tango.sardana.macro import createMacroNode
+
 _BASE_DIR = os.path.dirname(os.path.abspath(__file__))
 
 
@@ -120,6 +125,14 @@ def is_macro(macro, abs_file=None, logger=None):
         return False
     return True
 
+def recur_map(fun, data):
+    """Recursive map. Similar to map, but maintains the list objects structure
+    """
+    if hasattr(data, "__iter__"):
+        return [recur_map(fun, elem) for elem in data]
+    else:
+        return fun(data)
+
 
 class MacroManager(MacroServerManager):
 
@@ -216,7 +229,7 @@ class MacroManager(MacroServerManager):
 
     def _findMacroLibNames(self, path=None):
         path = path or self.getMacroPath()
-        ret = {}
+        ret = OrderedDict()
         for p in reversed(path):
             try:
                 for f in os.listdir(p):
@@ -440,14 +453,20 @@ class MacroManager(MacroServerManager):
             path = copy.copy(path)
             path.reverse()
 
+        mod_manager = ModuleManager()
+        m, exc_info = None, None
+        valid, exc_info = mod_manager.isValidModule(module_name, path)
+        if not valid:
+            params = dict(module=m, name=module_name,
+                          macro_server=self.macro_server, exc_info=exc_info)
+            return MacroLibrary(**params)
+
         # if there was previous Macro Library info remove it
         old_macro_lib = self._modules.pop(module_name, None)
         if old_macro_lib is not None:
             for macro in old_macro_lib.get_macros():
                 self._macro_dict.pop(macro.name)
 
-        mod_manager = ModuleManager()
-        m, exc_info = None, None
         try:
             m = mod_manager.reloadModule(module_name, path)
         except:
@@ -631,14 +650,32 @@ class MacroManager(MacroServerManager):
             macro_meta = self.getMacro(macro_name)
             ret.append(json_codec.encode(('', macro_meta.serialize()))[1])
         return ret
+    
+    def _createMacroNode(self, macro_name, macro_params):
+        macro = self.getMacro(macro_name)
+        params_def = macro.get_parameter()
+        return createMacroNode(macro_name, params_def, macro_params)
 
-    def decodeMacroParameters(self, door, in_par_list):
-        if len(in_par_list) == 0:
-            raise RuntimeError('Macro name not specified')
-        macro_name = in_par_list[0]
+    def decodeMacroParameters(self, door, raw_params):
+        """Decode macro parameters
+
+        :param door: (sardana.macroserver.msdoor.MSDoor) door object
+        :param raw_params: (lxml.etree._Element or list) xml element representing
+                          macro with subelements representing parameters or list
+                          with macro name followed by parameter values
+        """
+        if isinstance(raw_params, etree._Element):
+            macro_name = raw_params.get("name")
+        elif isinstance(raw_params, list):
+            # leave only macro parameters in the list
+            macro_name = raw_params.pop(0)
+        else:
+            raise Exception("Wrong format of raw_params object")
         macro_meta = self.getMacro(macro_name)
-        out_par_list = ParamDecoder(door, macro_meta, in_par_list)
-        return macro_meta, in_par_list, out_par_list
+        params_def = macro_meta.get_parameter()
+        type_manager = door.type_manager
+        out_par_list = ParamDecoder(type_manager, params_def, raw_params)
+        return macro_meta, raw_params, out_par_list
 
     def strMacroParamValues(self, par_list):
         """strMacroParamValues(list<string> par_list) -> list<string>
@@ -784,13 +821,18 @@ class MacroExecutor(Logger):
     def getNewMacroID(self):
         self._macro_counter -= 1
         return self._macro_counter
+    
+    def _createMacroNode(self, macro_name, macro_params):
+        return self.macro_manager._createMacroNode(macro_name, macro_params)
 
     def _preprocessParameters(self, par_str_list):
         if not par_str_list[0].lstrip().startswith('<'):
             xml_root = xml_seq = etree.Element('sequence')
-            xml_macro = etree.SubElement(xml_seq, 'macro', name=par_str_list[0])
-            for p in par_str_list[1:]:
-                etree.SubElement(xml_macro, 'param', value=p)
+            macro_name = par_str_list[0]
+            macro_params = par_str_list[1:]
+            macro_node = self._createMacroNode(macro_name, macro_params)
+            xml_macro = macro_node.toXml()
+            xml_seq.append(xml_macro)
         else:
             xml_root = etree.fromstring(par_str_list[0])
 
@@ -815,25 +857,6 @@ class MacroExecutor(Logger):
             if eid is None:
                 eid = str(self.getNewMacroID())
                 macro.set('id', eid)
-            name = macro.get('name')
-            params = []
-            
-            # SEEMS THERE IS A MEMORY LEAK IN lxml.etree Element.xpath :
-            # https://bugs.launchpad.net/lxml/+bug/397933
-            # https://mailman-mail5.webfaction.com/pipermail/lxml/2011-October/006205.html
-            # We work around it using findall:
-            
-            #for p in macro.xpath('param|paramrepeat'):
-            #    if p.tag == 'param':
-            #        params.append(p.get('value'))
-            #    else:
-            #        params.extend([ p2.get('value') for p2 in p.findall(".//param")])
-            for p in macro.findall('*'):
-                if p.tag == 'param':
-                    params.append(p.get('value'))
-                elif p.tag == 'paramrepeat':
-                    params.extend([ p2.get('value') for p2 in p.findall(".//param")])
-            macro.set('macro_line', "%s(%s)" % (name, ", ".join(params)))
 
     def __preprocessResult(self, result):
         """decodes the given output from a macro in order to be able to send to
@@ -851,24 +874,32 @@ class MacroExecutor(Logger):
             result = (str(result),)
         return result
 
-    def _decodeXMLMacroParameters(self, xml_macro):
-        str_params = [xml_macro.get('name')]
-        for param in xml_macro.findall('.//param'):
-            str_params.append(param.get('value'))
-        return self._decodeMacroParameters(str_params)
-
     def _decodeMacroParameters(self, params):
         return self.macro_manager.decodeMacroParameters(self.door, params)
+    
+    def _composeMacroLine(self, macro_name, macro_params, macro_id):
+        # recursive map to maintain the list objects structure
+        params_str_list = recur_map(str, macro_params)
+        # plain map to be able to perform join (only strings may be joined)
+        params_str_list = map(str, params_str_list)
+        params_str = ', '.join(params_str_list)
+        macro_id = macro_id
+        # create macro_line - string representation of macro, its parameters and id
+        macro_line = "%s(%s) -> %s" % (macro_name, params_str, macro_id)
+        return macro_line
 
     def _prepareXMLMacro(self, xml_macro, parent_macro=None):
-        macro_meta, _, params = self._decodeXMLMacroParameters(xml_macro)
+        macro_meta, _, macro_params = self._decodeMacroParameters(xml_macro)
+        macro_name = macro_meta.name
+        macro_id = xml_macro.get("id")
+        macro_line = self._composeMacroLine(macro_name, macro_params, macro_id)
         init_opts = {
-            'id'           : xml_macro.get('id'),
-            'macro_line'   : xml_macro.get('macro_line'),
+            'id'           : macro_id,
+            'macro_line'   : macro_line,
             'parent_macro' : parent_macro,
         }
 
-        macro_obj = self._createMacroObj(macro_meta, params, init_opts)
+        macro_obj = self._createMacroObj(macro_meta, macro_params, init_opts)
         for macro in xml_macro.findall('macro'):
             hook = MacroExecutor.RunSubXMLHook(self, macro)
             hook_hints = macro.findall('hookPlace')
@@ -877,7 +908,7 @@ class MacroExecutor(Logger):
             else:
                 hook_places = [ h.text for h in hook_hints ]
                 macro_obj.hooks = [ (hook, hook_places) ]
-        prepare_result = self._prepareMacroObj(macro_obj, params)
+        prepare_result = self._prepareMacroObj(macro_obj, macro_params)
         return macro_obj, prepare_result
 
     def _createMacroObj(self, macro_name_or_meta, pars, init_opts={}):
@@ -922,16 +953,25 @@ class MacroExecutor(Logger):
            Several different parameter formats are supported:
            1. several parameters:
              1.1 executor.prepareMacro('ascan', 'th', '0', '100', '10', '1.0')
+                 executor.prepareMacro('mv', [['th', '0']])
              1.2 executor.prepareMacro('ascan', 'th', 0, 100, 10, 1.0)
+                 executor.prepareMacro('mv', [['th', 0]])
              1.3 th = self.getObj('th');
                  executor.prepareMacro('ascan', th, 0, 100, 10, 1.0)
+                 executor.prepareMacro('mv', [[th, 0]])
            2. a sequence of parameters:
               2.1 executor.prepareMacro(['ascan', 'th', '0', '100', '10', '1.0')
+                  executor.prepareMacro(['mv', [['th', '0']]])
               2.2 executor.prepareMacro(('ascan', 'th', 0, 100, 10, 1.0))
+                  executor.prepareMacro(['mv', [['th', 0]]])
               2.3 th = self.getObj('th');
                   executor.prepareMacro(['ascan', th, 0, 100, 10, 1.0])
-           3. a space separated string of parameters:
+                  executor.prepareMacro(['mv', [[th, 0]]])
+           3. a space separated string of parameters (this is not compatible
+              with multiple or nested repeat parameters, furthermore the repeat
+              parameter must be the last one):
               executor.prepareMacro('ascan th 0 100 10 1.0')
+              executor.prepareMacro('mv %s 0' % motor.getName())
 
         :param pars: the command parameters as explained above
         :param opts: keyword optional parameters for prepare
@@ -940,17 +980,33 @@ class MacroExecutor(Logger):
         par0 = pars[0]
         if len(pars) == 1:
             if is_pure_str(par0):
-                pars = par0.split(' ')
+                # dealing with sth like args = ('ascan th 0 100 10 1.0',)
+                pars = par0.split()
+                macro_name, macro_params = pars[0], pars[1:]
+                macro_node = self._createMacroNode(macro_name, macro_params)
+                pars = macro_node.toList()
             elif is_non_str_seq(par0):
+                # dealing with sth like args = (['ascan', 'th', '0', '100', '10', '1.0'],)
+                # or args = (['mv', [[mot01, 0], [mot02, 0]]])
                 pars = par0
-        pars = map(str, pars)
+        # dealing with sth like args = ('ascan', 'th', '0', '100', '10', '1.0')
+        # or args = ('mv', [[mot01, 0], [mot02, 0]])
 
-        macro_klass, str_pars, pars = self._decodeMacroParameters(pars)
+        # in case parameters were passed as objects cast them to strings
+        pars = recur_map(str, pars)
 
-        init_opts['macro_line'] = "%s(%s) -> [%s]" % (str_pars[0], ", ".join(str_pars[1:]), id)
-        if not init_opts.has_key('id'):
-            init_opts['id'] = str(self.getNewMacroID())
-        return self.prepareMacroObj(macro_klass, pars, init_opts, prepare_opts)
+        meta_macro, _, macro_params = self._decodeMacroParameters(pars)
+        macro_name = meta_macro.name
+        macro_id = init_opts.get("id")
+        if macro_id is None:
+            macro_id = str(self.getNewMacroID())
+            init_opts["id"] = macro_id
+        macro_line = self._composeMacroLine(macro_name, macro_params, macro_id)
+
+        init_opts['macro_line'] = macro_line
+
+        return self.prepareMacroObj(meta_macro, macro_params, init_opts, 
+                                    prepare_opts)
 
     def getRunningMacro(self):
         return self._macro_pointer
@@ -1074,10 +1130,10 @@ class MacroExecutor(Logger):
         try:
             self.__runStatelessXML()
             self.sendState(Macro.Finished)
-        except AbortException:
+        except (StopException, AbortException):
             self.sendState(Macro.Abort)
         except Exception:
-            self.sendState(Macro.Abort)
+            self.sendState(Macro.Exception)
         finally:
             self._macro_stack = None
             self._xml_stack = None
