@@ -61,18 +61,25 @@ from IPython.core.profiledir import ProfileDirError, ProfileDir
 from IPython.core.application import BaseIPythonApplication
 from IPython.core.interactiveshell import InteractiveShell
 from IPython.utils.io import ask_yes_no as _ask_yes_no
-from IPython.utils.io import raw_input_ext as _raw_input_ext
-from IPython.utils.path import get_ipython_dir
 from IPython.utils.process import arg_split
 from IPython.utils.coloransi import TermColors
-from IPython.config.application import Application
-from IPython.frontend.terminal.ipapp import TerminalIPythonApp, \
-    launch_new_instance
+from IPython.terminal.ipapp import TerminalIPythonApp, launch_new_instance
 
-from taurus.core.taurushelper import Factory, Manager, Warning
+try:
+    # IPython 4.x
+    from traitlets.config.application import Application
+    from IPython.paths import get_ipython_dir
+except:
+    # IPython <4.x
+   from IPython.config.application import Application
+   from IPython.utils.path import get_ipython_dir
+
+
+import taurus
+#from taurus.core import Release as TCRelease
+
+from taurus.core.taurushelper import Factory
 from taurus.core.util.codecs import CodecFactory
-from taurus.core.taurushelper import setLogLevel
-
 
 # make sure Qt is properly initialized
 from taurus.external.qt import Qt
@@ -140,7 +147,7 @@ def ask_yes_no(prompt,default=None):
     return _ask_yes_no(prompt, default)
 
 def spock_input(prompt='',  ps2='... '):
-    return _raw_input_ext(prompt=prompt, ps2=ps2)
+    return raw_input(prompt)
 
 def translate_version_str2int(version_str):
     """Translates a version string in format x[.y[.z[...]]] into a 000000 number"""
@@ -410,7 +417,7 @@ def from_name_to_tango(name):
 #-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-~-
 
 def clean_up():
-    Manager().cleanUp()
+    taurus.Manager().cleanUp()
 
 def get_taurus_core_version():
     try:
@@ -598,7 +605,14 @@ def create_spock_profile(userdir, dft_profile, profile, door_name=None):
     if not os.path.isdir(userdir):
         ProfileDir.create_profile_dir(userdir)
     p_dir = ProfileDir.create_profile_dir_by_name(userdir, profile)
-    config_file_name = BaseIPythonApplication.config_file_name.default_value
+    ###########################################################################
+    # NOTE: BaseIPythonApplication.config_file_name.default_value should return
+    # the config file name, but it returns an empty string instead (at least 
+    # in some cases). For now, we give a hardcoded name if it is empty
+    # TODO: Check why this is the case
+    config_file_name = BaseIPythonApplication.config_file_name.default_value 
+    config_file_name = config_file_name or 'ipython_config.py' 
+    ###########################################################################
     abs_config_file_name = os.path.join(p_dir.location, config_file_name)
     create_config = True
     if os.path.isfile(abs_config_file_name):
@@ -653,7 +667,6 @@ config.IPKernelApp.pylab = 'inline'
     sys.stdout.write('Storing %s in %s... ' % (config_file_name, p_dir.location))
     sys.stdout.flush()
 
-
     with file(abs_config_file_name, "w") as f:
         f.write(dest_data)
         f.close()
@@ -788,6 +801,7 @@ def load_ipython_extension(ipython):
     init_taurus()
 
     config = ipython.config
+
     user_ns = ipython.user_ns
     user_ns['MACRO_SERVER_NAME'] = config.Spock.macro_server_name
     user_ns['MACRO_SERVER_ALIAS'] = config.Spock.macro_server_alias
@@ -939,12 +953,6 @@ object?   -> Details about 'object'. ?object also works, ?? prints more.
     completer.omit__names = 2
     completer.greedy = False
 
-    # ------------------------------------
-    # InteractiveShellApp
-    # ------------------------------------
-    i_shell_app = config.InteractiveShellApp
-    i_shell_app.ignore_old_config = True
-
     # ------------------------------------
     # TerminalIPythonApp: options for the IPython terminal (and not Qt Console)
     # ------------------------------------
@@ -1032,7 +1040,7 @@ object?   -> Details about 'object'. ?object also works, ?? prints more.
 def start(user_ns=None):
     # Make sure the log level is changed to warning
     CodecFactory()
-    setLogLevel(Warning)
+    taurus.setLogLevel(taurus.Warning)
 
     try:
         check_requirements()
@@ -1119,10 +1127,19 @@ def prepare_cmdline(argv=None):
 
 
 def run():
-
     try:
-        from IPython.utils.traitlets import Unicode
-        from IPython.frontend.qt.console.rich_ipython_widget import RichIPythonWidget
+        try:
+            # IPython 4.x
+            from traitlets import Unicode
+            from qtconsole.rich_jupyter_widget import RichIPythonWidget
+            from qtconsole.qtconsoleapp import IPythonQtConsoleApp
+            # TODO: check if we can/should set IPythonQtConsoleApp.version
+        except:
+            # IPython <4.x
+            from IPython.utils.traitlets import Unicode
+            from IPython.qt.console.rich_ipython_widget import RichIPythonWidget
+            from IPython.qt.console.qtconsoleapp import IPythonQtConsoleApp
+            IPythonQtConsoleApp.version.default_value = release.version
         
         class SpockConsole(RichIPythonWidget):
 
@@ -1132,10 +1149,8 @@ def run():
                 config = get_config()
                 return config.FrontendWidget.banner
 
-        import IPython.frontend.qt.console.qtconsoleapp
-        IPythonQtConsoleApp = IPython.frontend.qt.console.qtconsoleapp.IPythonQtConsoleApp
         IPythonQtConsoleApp.widget_factory = SpockConsole
-        IPythonQtConsoleApp.version.default_value = release.version
+
     except ImportError:
         pass
 
