@@ -1237,6 +1237,9 @@ class MGConfiguration(object):
                 except:
                     pass
 
+    def getChannels(self):
+        return self.channel_list
+
     def getChannelInfo(self, channel_name):
         try:
             return self.tango_channels_info[channel_name]
@@ -1395,7 +1398,7 @@ class MeasurementGroup(PoolElement):
         self.write_attribute("configuration", json.dumps(cfg))
 
     def getChannels(self):
-        return self.getConfiguration().channel_list
+        return self.getConfiguration().getChannels()
 
     def getCounters(self):
         cfg = self.getConfiguration()
@@ -1443,6 +1446,42 @@ class MeasurementGroup(PoolElement):
         self._last_integ_time = ctime
         self.getIntegrationTimeObj().write(ctime)
 
+    def enableChannels(self, channels):
+        '''Enable acquisition of the indicated channels.
+
+        :param channels: (seq<str>) a sequence of strings indicating
+                         channel names
+        '''
+        self._enableChannels(channels, True)
+
+    def disableChannels(self, channels):
+        '''Disable acquisition of the indicated channels.
+
+        :param channels: (seq<str>) a sequence of strings indicating
+                         channel names
+        '''
+        self._enableChannels(channels, False)
+
+    def _enableChannels(self, channels, state):
+        found = {}
+        for channel in channels:
+            found[channel] = False
+        cfg = self.getConfiguration()
+        for channel in cfg.getChannels():
+            name = channel['name']
+            if name in channels:
+                channel['enabled'] = state
+                found[name] = True
+        wrong_channels = []
+        for ch, f in found.items():
+            if f is False:
+                wrong_channels.append(ch)
+        if len(wrong_channels) > 0:
+            msg = 'channels: %s are not present in measurement group' % \
+                                                                wrong_channels
+            raise Exception(msg)
+        self.setConfiguration(cfg.raw_data)
+
     def _start(self, *args, **kwargs):
         self.Start()
 
@@ -1583,14 +1622,28 @@ class Pool(TangoDevice, MoveableSource):
             element = BaseSardanaElement(**element_data)
             elements.addElement(element)
         for element_data in elems.get('del', ()):
-            element = self.getElementInfo(element_data['name'])
+            element = self.getElementInfo(element_data['full_name'])
             try:
                 elements.removeElement(element)
             except:
                 self.warning("Failed to remove %s", element_data)
-
+        for element_data in elems.get('change', ()):
+            element = self._removeElement(element_data)
+            element = self._addElement(element_data)
         return elems
 
+    def _addElement(self, element_data):
+        element_data['manager'] = self
+        element = BaseSardanaElement(**element_data)
+        self.getElementsInfo().addElement(element)
+        return element
+
+    def _removeElement(self, element_data):
+        name = element_data['full_name']
+        element = self.getElementInfo(name)
+        self.getElementsInfo().removeElement(element)
+        return element
+
     def getElementsInfo(self):
         return self._elements
 
@@ -1743,6 +1796,14 @@ class Pool(TangoDevice, MoveableSource):
         elements_info = self.getElementsInfo()
         return self._wait_for_element_in_container(elements_info, name)
 
+    def renameElement(self, old_name, new_name):
+        self.debug('trying to rename element: %s to: %s', old_name, new_name)
+        self.command_inout('RenameElement', [old_name, new_name])
+        elements_info = self.getElementsInfo()
+        return self._wait_for_element_in_container(elements_info, new_name,
+                                                   contains=True)
+
+
     def deleteElement(self, name):
         self.debug('trying to delete element: %s', name)
         self.command_inout('DeleteElement', name)
